unit WDosControlEx;

interface

procedure Register;

implementation

uses
  Windows, Messages, SysUtils, Classes, DsgnIntf, Controls, Dialogs, Forms,
  WDosPeas, WDosPeasInOuts, WDosEpcs, WDosEpcInOuts, WDosPneumatics,
  WDosWatchdogs, ProtocolEditDlgs;

type
  TProtocolProperty = class(TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
    procedure Edit; override;
  end;

  TwdxEpcCtrlEditor = class(TComponentEditor)
  protected
    procedure EditProtocolProperty;
  public
    function GetVerbCount: Integer; override;
    function GetVerb(Index: Integer): string; override;
    procedure ExecuteVerb(Index: Integer); override;
  end;

procedure Register;
begin
  { register control components }
  RegisterComponents('WDosControl', [TwdxPeasIo, TwdxPeasCtrl, TwdxPeasInOut,
      TwdxEpcInOut, TwdxCylinder]);
  RegisterComponents('WDosSystem', [TwdxArWatchDog, TwdxEpcCtrl]);
  RegisterPropertyEditor( TypeInfo(TProtocolLength), nil, '',
      TProtocolProperty);
  RegisterComponentEditor(TwdxEpcCtrl, TwdxEpcCtrlEditor);
  RegisterComponentEditor(TwdxEpcInOut, TwdxEpcCtrlEditor);
end;

{
****************************** TProtocolProperty *******************************
}
procedure TProtocolProperty.Edit;
var
  Dialog: TProtocolEditDlg;
  lSlaves: TSlaves;
  lProtocol: TProtocolLength;
  Persistent: TPersistent;
begin
  Dialog :=TProtocolEditDlg.Create(Application);
  try
    Persistent :=GetComponent(0);
    if Persistent is TwdxEpcCtrl then
    begin
      lSlaves :=TwdxEpcCtrl(Persistent).Slaves;
      lProtocol :=TProtocolLength(GetOrdValue);
      with Dialog do
      begin
        Protocol :=lProtocol;
        Slaves :=lSlaves;
        if ShowModal = mrOK then SetOrdValue(Protocol);
      end;
    end
    else if Persistent is TwdxEpcInOut then
    begin
      lSlaves :=TwdxEpcInOut(Persistent).Slaves;
      lProtocol :=TProtocolLength(GetOrdValue);
      with Dialog do
      begin
        Protocol :=lProtocol;
        Slaves :=lSlaves;
        if ShowModal = mrOK then SetOrdValue(Protocol);
      end;
    end
    else ShowMessage(
        Format('Property editor for %s not registered',
        [Persistent.ClassName]));
  finally
    Dialog.Free;
  end;
end;

function TProtocolProperty.GetAttributes: TPropertyAttributes;
begin
  Result :=[paDialog, paRevertable];
end;

function TProtocolProperty.GetValue: string;
begin
  Result :='$' +IntToHex(GetOrdValue, 8);
end;

procedure TProtocolProperty.SetValue(const Value: string);
var
  lValue: LongWord;
begin
  lValue :=StrToInt(Value) and $3FFFFFFF;
  SetOrdValue(lValue);
end;

{
****************************** TwdxEpcCtrlEditor *******************************
}
function TwdxEpcCtrlEditor.GetVerbCount: Integer;
begin
  Result :=1;
end;

function TwdxEpcCtrlEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result :='Suconet-K1 protocol editor';
  end; { case }
end;

procedure TwdxEpcCtrlEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0: EditProtocolProperty;
  end; { case }
end;

procedure TwdxEpcCtrlEditor.EditProtocolProperty;
begin
  with TProtocolEditDlg.Create(Application) do
  try
    if Component is TwdxEpcCtrl then
    begin
      Slaves :=TwdxEpcCtrl(Component).Slaves;
      Protocol :=TwdxEpcCtrl(Component).Protocol;
      if ShowModal = mrOK then
      begin
        TwdxEpcCtrl(Component).Protocol :=Protocol;
        Self.Designer.Modified;
      end;
    end
    else if Component is TwdxEpcInOut then
    begin
      Slaves :=TwdxEpcInOut(Component).Slaves;
      Protocol :=TwdxEpcInOut(Component).Protocol;
      if ShowModal = mrOK then
      begin
        TwdxEpcInOut(Component).Protocol :=Protocol;
        Self.Designer.Modified;
      end;
    end
    else ShowMessage(
        Format('Property editor for %s not registered',
        [Component.ClassName]));
  finally
    Free;
  end;
end;


end.
